-- File: archive\db\sql\save\twh_getdeletedlob.sql
-- SID: 1.3
-- Date Modified: 02/01/30 15:14:20
--
-- Module Description
--   Defines the twh_getdeletedlob stored procedure

USE Object
GO

---------------------------------------------------------------------------------------------------
-- twh_getdeletedlob
PRINT 'Creating procedure twh_getdeletedlob'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'twh_getdeletedlob' AND type = 'P')
  DROP PROCEDURE twh_getdeletedlob
GO

-- File: archive\db\sql\save\twh_getdeletedlob.sql
-- SID: 1.3
-- Date Modified: 02/01/30 15:14:20
/*
Gets all LOBs that have been deleted since the last load to the warehouse.
Returns the following:
  sourceid: The parent lob of the deleted lob, null for top (lob container)
  targetid: The lob being deleted.  LOBs that are pointers to existing LOBs will show the links lobid
      and will have the softflag (below) set to 1.  Note that the warehouse stores these as one
      lob with several parents.  This allows us to delete only the links when appropriate.
  softflag: 1 or 0, indicates that the deleted lob was an LOBL pointer to targetid above.
  deldate:  The deletion date of the lob (mtime).
The id column of the temp table is the true id of the LOB
*/
CREATE PROCEDURE twh_getdeletedlob
AS
  SET NOCOUNT ON
  --Get GM time offset
  DECLARE @gmoffsetmin INT

  EXEC master..xp_regread 
    'HKEY_LOCAL_MACHINE',
    'SYSTEM\CurrentControlSet\Control\TimeZoneInformation',
    'ActiveTimeBias', 
     @gmoffsetmin OUTPUT

  --First reset deletions with deletion in-process status (deletions that didn't get all the way into the warehouse)
  UPDATE twh_lob SET status=status-4, deldate=NULL WHERE status&4=4

  CREATE TABLE #dellob
  (
    id ObjID,
    sourceid ObjID NULL,
    targetid ObjID NULL,
    softflag TINYINT,
    deldate DATETIME
  )

  --Load temp table with lobs deleted since last load
  INSERT INTO #dellob
    SELECT o.id,
      CASE l.src_cid WHEN 'LOB' THEN l.src_id ELSE NULL END AS sourceid,
      ISNULL(t.linkedid,o.id) AS targetid,
      CASE WHEN t.linkedid IS NULL THEN 0 ELSE 1 END AS softflag,
      o.mtime AS deldate
    FROM LineOfBusiness_V o, twh_lob t, link l
    WHERE o.id=t.id AND t.deldate IS NULL AND l.link_type='LOBC' AND l.dst_cid='LOB' AND l.dst_id=o.id AND l.deleted=1

  --Update twh_lob table.  This sets the deldate, and also sets the status bit for deletion in process
  UPDATE t SET deldate=d.deldate, status=status+4 FROM twh_lob t, #dellob d WHERE d.id=t.id

  --Return data to the warehouse
  SELECT sourceid, NULL AS comp_source_id, targetid, NULL AS comp_target_id, softflag, DATEADD(mi, @gmoffsetmin, deldate) AS deldate FROM #dellob
GO
