-- File: archive\db\sql\save\twh_getlobprcstate.sql
-- SID: 1.7
-- Date Modified: 02/08/27 15:15:08
--
-- Module Description
--   Defines the twh_getlobprcstate stored procedure

USE Object
GO

---------------------------------------------------------------------------------------------------
-- twh_getlobprcstate
PRINT 'Creating procedure twh_getlobprcstate'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'twh_getlobprcstate' AND type = 'P')
  DROP PROCEDURE twh_getlobprcstate
GO


-- File: archive\db\sql\save\twh_getlobprcstate.sql
-- SID: 1.7
-- Date Modified: 02/08/27 15:15:08
/*
Gets hourly LOB state percentages since the last load to the warehouse
Returns the following:
  id: The TBSM id of the LOB.  This will never be an LOB that is a link to an existing LOB.
  msmttyp_nm: The Measurement type name as stored in the CDW.  This will be derived here from twh_alertstate
  msmt_strt_dt: The date in the form yyyy-mm-dd, based on GM Time
  msmt_strt_tm: The time in the form hh:mm:ss, always the beginning of the whole hour, based on GM Time
  msmt_avg_val: The percentage of the hour in the above state
*/
CREATE PROCEDURE twh_getlobprcstate
AS
  SET NOCOUNT ON
  DECLARE @lastenddate DATETIME, @thisenddate DATETIME, @dummy INT, @tempdate DATETIME, @cnt INT, @daysold DECIMAL(8,0)

  --Get GM time offset
  DECLARE @gmoffsetmin INT

  EXEC master..xp_regread 
    'HKEY_LOCAL_MACHINE',
    'SYSTEM\CurrentControlSet\Control\TimeZoneInformation',
    'ActiveTimeBias', 
     @gmoffsetmin OUTPUT

  --Reverse tracking tables for measurements that didn't get all the way to the CDW
  UPDATE twh_lob SET status=status-8, lastalertstate=prevlastalertstate, prevlastalertstate=NULL WHERE status&8=8
  UPDATE twh_date SET status=status-1, lastenddate=prevlastenddate, prevlastenddate=NULL
    WHERE status&1=1 AND loadtype='lobstate'

  --Get last cutoff (ending) date if it exists
  SELECT @lastenddate=lastenddate FROM twh_date WHERE status&1=0 AND loadtype='lobstate'

  --If no last cutoff date exists (ie the first load) check for days to keep sent from twh
  --and calculate cutoff date going back that many days and rounding to last whole hour.
  IF @lastenddate IS NULL
  BEGIN
    SELECT @daysold=pmsmtc_age_in_days FROM twh_prune_msmt_control WHERE tmsum_cd='H'
    IF NOT @daysold IS NULL
    BEGIN
      SET @lastenddate=DATEADD(d,-@daysold,CURRENT_TIMESTAMP)
      SET @lastenddate=
          DATEADD(ms,
         -DATEDIFF(ms,
           CONVERT(DATETIME,CONVERT(NVARCHAR(2),DATEPART(hh,@lastenddate))+':00:00.000',114),
           CONVERT(DATETIME,CONVERT(NVARCHAR(12),@lastenddate,114),114)
         ),
       @lastenddate)
    END
  END

  --Get this cutoff using last whole hour
  SET @thisenddate=CURRENT_TIMESTAMP
  SET @thisenddate=
  DATEADD(ms,
    -DATEDIFF(ms,
         CONVERT(DATETIME,CONVERT(NVARCHAR(2),DATEPART(hh,@thisenddate))+':00:00.000',114),
         CONVERT(DATETIME,CONVERT(NVARCHAR(12),@thisenddate,114),114)
    ),
  @thisenddate)

  --Summarize lobs to be processed along with their date ranges
  CREATE TABLE #templob (lobid INT, mindate DATETIME, maxdate DATETIME)
  INSERT INTO #templob
    SELECT t.id,
    CASE
      WHEN @lastenddate IS NULL THEN l.ctime
      WHEN l.ctime>@lastenddate THEN l.ctime
         ELSE @lastenddate
       END,
    CASE
      WHEN t.deldate IS NULL THEN @thisenddate
         WHEN t.deldate<@thisenddate THEN t.deldate
         ELSE @thisenddate
       END
       FROM twh_lob t, LineOfBusiness_V l
      WHERE l.id=t.id AND t.linkedid IS NULL
      AND (t.deldate IS NULL OR (@lastenddate IS NULL OR t.deldate>=@lastenddate))
      AND l.ctime<=@thisenddate

  --Initialize any null lastalertstates on twh_lob.  This addresses the case where a date has been
  --manually entered to avoid getting all past history by looking up the last state change on AlertStateHistory
  --prior to the beginning date, and using 1 if nothing is found.
  UPDATE tl SET lastalertstate=
    ISNULL((SELECT AlertStateID FROM AlertStateHistory WHERE cid='LOB' AND id=t.lobid AND ctime=
      (SELECT MAX(ctime) FROM AlertStateHistory WHERE cid='LOB' AND id=t.lobid AND ctime<t.mindate)
    ),1)
  FROM twh_lob tl, #templob t WHERE tl.id=t.lobid AND tl.lastalertstate IS NULL

  --Create temporary table to accumulate all state changes and whole hour markers
  CREATE TABLE #tempstate (lobid INT, dt DATETIME, state INT NULL, seq INT NULL PRIMARY KEY CLUSTERED(lobid, dt))

  --Add appropriate data from AlertStateHistory
  SELECT t.lobid AS lobid, a.ctime AS dt, a.AlertStateID AS state INTO #tempstate_temp
    FROM #templob t, AlertStateHistory a
      WHERE a.cid='LOB' AND a.id=t.lobid AND a.ctime>=t.mindate AND a.ctime<t.maxdate
      AND (a.AlertStateID != a.OldAlertStateID OR a.OldAlertStateID IS NULL)
  -- delete low alertstate douplicate
  DELETE temp1 FROM #tempstate_temp AS temp1 
    JOIN #tempstate_temp AS temp2 ON temp1.lobid = temp2.lobid
    AND temp1.dt = temp2.dt
    AND temp1.state < temp2.state

  INSERT INTO #tempstate SELECT DISTINCT lobid, dt, state, NULL FROM #tempstate_temp 
  --Ensure that records exist for the mindates for all lobs.  This uses the last known state of the lob
  INSERT INTO #tempstate
  SELECT t.lobid, t.mindate, tl.lastalertstate, NULL
    FROM #templob t, twh_lob tl
    WHERE tl.id=t.lobid AND NOT EXISTS (SELECT 1 FROM #tempstate WHERE lobid=t.lobid AND dt=t.mindate)

  --Now add unknown state records for the previous whole hour before the mindate, only if the
  --mindate is not a whole hour.  This addresses the unknown portion of the whole hour for new lobs.
  INSERT INTO #tempstate
  SELECT
    t.lobid,
    DATEADD(ms,
      -DATEDIFF(ms,
           CONVERT(DATETIME,CONVERT(NVARCHAR(2),DATEPART(hh,t.mindate))+':00:00.000',114),
           CONVERT(DATETIME,CONVERT(NVARCHAR(12),t.mindate,114),114)
      ),
    t.mindate),
    0,
    NULL
  FROM #templob t WHERE NOT EXISTS
    (SELECT 1 FROM #tempstate WHERE lobid=t.lobid AND dt=
      DATEADD(ms,
        -DATEDIFF(ms,
             CONVERT(DATETIME,CONVERT(NVARCHAR(2),DATEPART(hh,t.mindate))+':00:00.000',114),
             CONVERT(DATETIME,CONVERT(NVARCHAR(12),t.mindate,114),114)
        ),
      t.mindate)
    )

  --Ensure that records exist for all maxdates.  Note that when the maxdate is  deletion datetime
  --an unknown state is used, otherwise it is set to null and updates sequentially below.
  INSERT INTO #tempstate
    SELECT
    t.lobid,
    t.maxdate,
    CASE
      WHEN t.maxdate=tl.deldate THEN 0 ELSE NULL
    END,
    NULL
    FROM #templob t, twh_lob tl
  WHERE t.lobid=tl.id AND NOT EXISTS (SELECT 1 FROM #tempstate WHERE lobid=t.lobid AND dt=t.maxdate)

  --Now add records for the next whole hour after the maxdate, only if the
  --maxdate is not a whole hour.  This addresses lobs deleted before this runs cutoff.
  INSERT INTO #tempstate
  SELECT
    t.lobid,
    DATEADD(hh,DATEPART(hh,t.maxdate)+
      CASE
        DATEDIFF(ms,
             CONVERT(DATETIME,CONVERT(NVARCHAR(2),DATEPART(hh,t.maxdate))+':00:00.000',114),
             CONVERT(DATETIME,CONVERT(NVARCHAR(12),t.maxdate,114),114)
        )
         WHEN 0 THEN 0 ELSE 1 END,
    CONVERT(DATETIME,CONVERT(NVARCHAR(10),t.maxdate,101),101)),
    NULL,
    NULL
     FROM #templob t WHERE NOT EXISTS
       (SELECT 1 FROM #tempstate WHERE lobid=t.lobid AND dt=
         DATEADD(hh,DATEPART(hh,t.maxdate)+
           CASE
          DATEDIFF(ms,
               CONVERT(DATETIME,CONVERT(NVARCHAR(2),DATEPART(hh,t.maxdate))+':00:00.000',114),
               CONVERT(DATETIME,CONVERT(NVARCHAR(12),t.maxdate,114),114)
          )
           WHEN 0 THEN 0 ELSE 1 END,
       CONVERT(DATETIME,CONVERT(NVARCHAR(10),t.maxdate,101),101))
    )

  --set up tables used to get cartesian product for adding all missing whole hours for each lob
  CREATE TABLE #days (day DATETIME)
  SELECT @tempdate=CONVERT(DATETIME,CONVERT(NVARCHAR(10),MIN(dt),101)) FROM #tempstate
  WHILE @tempdate<=@thisenddate
  BEGIN
    INSERT INTO #days VALUES(@tempdate)
    SET @tempdate=DATEADD(d,1,@tempdate)
  END
  CREATE TABLE #hours (hour INT)
  SET @cnt=0
  WHILE @cnt<24
  BEGIN
    INSERT INTO #hours VALUES(@cnt)
    SET @cnt=@cnt+1
  END

  --Insert missing whole hours
  INSERT INTO #tempstate (lobid, dt)
    SELECT l.lobid, DATEADD(hh, h.hour, d.day) FROM #templob l, #days d, #hours h WHERE NOT EXISTS
      (SELECT 1 FROM #tempstate WHERE lobid=l.lobid AND dt=DATEADD(hh, h.hour, d.day))
      AND DATEADD(hh, h.hour, d.day) > l.mindate
      AND DATEADD(hh, h.hour, d.day) < l.maxdate

  --Populate null alertstates with previous state.
  DECLARE @lobid ObjID, @holdlobid ObjID, @state ObjID, @holdstate ObjID
  SET @holdlobid=0
  SET @holdstate=NULL
  SET @cnt=0
  DECLARE lobstate CURSOR FOR SELECT lobid, state FROM #tempstate ORDER BY lobid, dt FOR UPDATE OF state, seq
  OPEN lobstate
  FETCH NEXT FROM lobstate INTO @lobid, @state
  WHILE @@FETCH_STATUS<>-1
  BEGIN
    SET @cnt=@cnt+1
    IF @lobid<>@holdlobid
    BEGIN
      SET @holdlobid=@lobid
      SET @holdstate=NULL
    END
    IF @state IS NULL
      UPDATE #tempstate SET state=@holdstate, seq=@cnt WHERE CURRENT OF lobstate
    ELSE
    BEGIN
      SET @holdstate=@state
      UPDATE #tempstate SET seq=@cnt WHERE CURRENT OF lobstate
    END
    FETCH NEXT FROM lobstate INTO @lobid, @state
  END
  DEALLOCATE lobstate

  --Index the seq number

  CREATE INDEX temp_state_seq ON #tempstate (seq)

  --Update tracking tables.  twh_date records this end date as the lastenddate in an in-process status.
  --This also sets the last alertstate on twh_lob to the most recent reported for each lob.
  BEGIN TRANSACTION
    UPDATE twh_date SET status=status+1, prevlastenddate=lastenddate, lastenddate=@thisenddate
      WHERE loadtype='lobstate' AND status&1=0
    IF @@ROWCOUNT<1
      INSERT twh_date (loadtype, lastenddate, status) VALUES ('lobstate',@thisenddate,1)
    UPDATE tl SET prevlastalertstate=lastalertstate, status=status+8, lastalertstate=
      (SELECT state FROM #tempstate WHERE lobid=t.lobid AND dt=
        DATEADD(hh,DATEPART(hh,t.maxdate)+
          CASE
            DATEDIFF(ms,
                 CONVERT(DATETIME,CONVERT(NVARCHAR(2),DATEPART(hh,t.maxdate))+':00:00.000',114),
                 CONVERT(DATETIME,CONVERT(NVARCHAR(12),t.maxdate,114),114)
            )
            WHEN 0 THEN 0 ELSE 1 END,
           CONVERT(DATETIME,CONVERT(NVARCHAR(10),t.maxdate,101),101))
      )
    FROM twh_lob tl, #templob t WHERE tl.id=t.lobid
  COMMIT TRANSACTION

  --This returns the percentage data to the warehouse.

  SELECT
    t1.lobid AS id,
    a.name AS msmttyp_nm,
    CONVERT(VARCHAR(10),
      CONVERT(DATETIME,
           CONVERT(VARCHAR(10),DATEADD(mi,@gmoffsetmin,t1.dt),120)
        +' '+CONVERT(VARCHAR(2),DATEPART(hh,DATEADD(mi,@gmoffsetmin,t1.dt)))+':00:00'
      ,120)
    ,120) AS msmt_strt_dt,
    CONVERT(VARCHAR(8),
      CONVERT(DATETIME,
           CONVERT(VARCHAR(10),DATEADD(mi,@gmoffsetmin,t1.dt),120)
        +' '+CONVERT(VARCHAR(2),DATEPART(hh,DATEADD(mi,@gmoffsetmin,t1.dt)))+':00:00'
      ,120)
    ,108) AS msmt_strt_tm,
    SUM(CONVERT(FLOAT,100)*CONVERT(FLOAT,DATEDIFF(ms,t1.dt,t2.dt))/CONVERT(FLOAT,3600000)) AS msmt_avg_val
  FROM #tempstate t1, #tempstate t2, twh_alertstate a WHERE t2.seq=t1.seq+1 AND t2.lobid=t1.lobid AND a.id=t1.state
  GROUP BY
    t1.lobid,
    a.name,
    CONVERT(VARCHAR(10),
      CONVERT(DATETIME,
           CONVERT(VARCHAR(10),DATEADD(mi,@gmoffsetmin,t1.dt),120)
        +' '+CONVERT(VARCHAR(2),DATEPART(hh,DATEADD(mi,@gmoffsetmin,t1.dt)))+':00:00'
      ,120)
    ,120),
    CONVERT(VARCHAR(8),
      CONVERT(DATETIME,
        CONVERT(VARCHAR(10),DATEADD(mi,@gmoffsetmin,t1.dt),120)
        +' '+CONVERT(VARCHAR(2),DATEPART(hh,DATEADD(mi,@gmoffsetmin,t1.dt)))+':00:00'
      ,120)
    ,108)
    ORDER BY t1.lobid, msmt_strt_dt, msmt_strt_tm

GO
