-- File: archive\db\sql\save\twh_getnewlob.sql
-- SID: 1.3
-- Date Modified: 02/01/30 15:14:22
--
-- Module Description
--   Defines the twh_getnewlob stored procedure

USE Object
GO

---------------------------------------------------------------------------------------------------
-- twh_getnewlob
PRINT 'Creating procedure twh_getnewlob'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'twh_getnewlob' AND type = 'P')
  DROP PROCEDURE twh_getnewlob
GO

-- File: archive\db\sql\save\twh_getnewlob.sql
-- SID: 1.3
-- Date Modified: 02/01/30 15:14:22
/*
Gets new LOBs added since the last load to the warehouse
Returns the following:
  sourceid: The parent lob of the new lob, null for top (lob container)
  targetid: The lob being added.  LOBs that are pointers to existing LOBs will show the links lobid
      and will have the softflag (below) set to 1.  Note that the warehouse stores these as one
      lob with several parents.  This allows us to add only the links when appropriate.
  softflag: 1 or 0, indicates that the added lob is an LOBL pointer to targetid above.
  adddate:  The add date of the lob (ctime).
  name:    The LOB name
  description: The LOB description
The id column of the temp table is the true id of the LOB
Note that currently this excludes LOBs that result from physical resources dragged into LOBs (and
any children that might exist below them).
*/
CREATE PROCEDURE twh_getnewlob
AS
  SET NOCOUNT ON
  --Get GM time offset
  DECLARE @gmoffsetmin INT

  EXEC master..xp_regread 
    'HKEY_LOCAL_MACHINE',
    'SYSTEM\CurrentControlSet\Control\TimeZoneInformation',
    'ActiveTimeBias', 
     @gmoffsetmin OUTPUT

  --First reset additions with addition in-process status (additions that didn't make it all the way to the warehouse)
  DELETE FROM twh_lob WHERE status&1=1

  --Now find all appropriate new LOBs
  CREATE TABLE #newlob
  (
    id ObjID,
    parentid ObjID NULL,
    ctime DATETIME
  )

  INSERT INTO #newlob
  SELECT o.id,
    CASE l.src_cid WHEN 'LOB' THEN src_id ELSE NULL END,
    o.ctime
  FROM LineOfBusiness_V o, link l WHERE l.link_type='LOBC' AND l.dst_cid='LOB' AND l.dst_id=o.id AND l.deleted=0
     AND NOT EXISTS (SELECT * FROM twh_lob WHERE id=o.id)

  --Delete anything from temp table that links to physical resouce
  DELETE FROM n FROM #newlob AS n WHERE EXISTS
    (SELECT * FROM link WHERE link_type='LOBL' AND src_cid='LOB' AND src_id=n.id AND dst_cid<>'LOB')

  --Recursively delete anything else that may have been linked to the above LOBs

  WHILE @@ROWCOUNT>0
    DELETE n FROM #newlob n WHERE NOT parentid IS NULL AND
      NOT EXISTS (SELECT * FROM #newlob WHERE id=n.parentid) AND
      NOT EXISTS (SELECT * FROM twh_lob WHERE id=n.parentid)
  --Insert new records into twh_lob in an addition in-process status.  Note that lastalertstate
  --is left null to allow proper initialization in twh_getlobprcstate.
  INSERT INTO twh_lob(id, linkedid, name, description, status)
  SELECT 
    o.id,
    (SELECT dst_id FROM link WHERE link_type='LOBL' AND src_cid='LOB' AND src_id=o.id AND dst_cid='LOB'),
    o.name, o._Desc, 1
     FROM #newlob n, LineOfBusiness_V o, link l
    WHERE n.id=o.id AND l.link_type='LOBC' AND l.dst_cid='LOB' AND l.dst_id=o.id AND l.deleted=0

  --Return records to the warehouse
  SELECT n.parentid AS sourceid,
       NULL AS comp_source_id,
    ISNULL(t.linkedid, n.id) AS targetid,
       NULL AS comp_target_id,
       CASE WHEN t.linkedid IS NULL THEN 0 ELSE 1 END AS softflag,
    DATEADD(mi, @gmoffsetmin, n.ctime) AS adddate,
    t.name,
    t.description
  FROM #newlob AS n, twh_lob AS t WHERE n.id=t.id
GO
